from enum import Enum

import pytest
from aidev_agent.utils.factory import GenericFactory, SimpleFactory, SingletonFactory

from tests._typing import FixtureType as ft


class MockType(Enum):
    Registered = "A"
    NotRegister = "B"


class TestGenericFactory:
    """测试 GenericFactory"""

    factory: GenericFactory[MockType, str]

    @pytest.fixture(autouse=True)
    def setup(self, faker: ft.faker):
        self.default_value = faker.word()
        self.registered_value = faker.word()

        self.factory = GenericFactory("testing", self.default_value)
        self.factory.register(MockType.Registered, self.registered_value)

    def test_without_defaults(self):
        """测试不传入默认值"""
        factory: GenericFactory[MockType, str] = GenericFactory("testing")

        with pytest.raises(NotImplementedError):
            factory.defaults.startswith("not-implemented!")

    def test_factory_type(self):
        """测试 factory_type 默认值"""

        assert self.factory.factory_type == "generic"

    def test_allow_overwrite(self):
        """测试 allow_overwrite 默认值"""

        assert self.factory.allow_overwrite is True

    def test_replace_defaults_allow_overwrite(self):
        """测试替换默认值"""

        self.factory.allow_overwrite = True

        default_value = "b"
        assert self.factory.replace_defaults(default_value) == self.default_value
        assert self.factory.get(MockType.NotRegister) == default_value

    def test_replace_defaults_overwrite(self):
        """测试不允许覆盖时替换默认值"""

        self.factory.allow_overwrite = False

        with pytest.raises(RuntimeError):
            self.factory.replace_defaults("u-raise-me-up")

    def test_register(self):
        """测试注册"""

        assert MockType.NotRegister not in self.factory

        self.factory.register(MockType.NotRegister, "b")

        assert MockType.NotRegister in self.factory

    def test_register_overwrite(self):
        """测试注册不允许覆盖"""

        self.factory.allow_overwrite = False

        with pytest.raises(RuntimeError):
            self.factory.register(MockType.Registered, "b")

    def test_register_allow_overwrite(self):
        """测试注册时允许覆盖"""

        self.factory.allow_overwrite = True

        self.factory.register(MockType.Registered, "b")

    def test_register_not_overwrite_because_of_priority(self):
        """测试因为优先级导致无法覆盖注册"""

        self.factory.register(MockType.Registered, "a", priority=1)
        self.factory.register(MockType.Registered, "b", priority=0)

        assert self.factory.get(MockType.Registered) == "a"

    def test_register_overwrite_because_of_priority(self):
        """测试优先级覆盖注册"""

        self.factory.register(MockType.Registered, "a", priority=0)
        self.factory.register(MockType.Registered, "b", priority=1)

        assert self.factory.get(MockType.Registered) == "b"

    def test_post_register(self):
        """测试注册后回调"""

        callbacks = {}

        class MockFactory(GenericFactory[MockType, str]):
            def _post_register(self, typ: MockType, value: str, priority: int):
                callbacks[typ] = value

        factory = MockFactory("testing")
        factory.register(MockType.Registered, "1")
        factory.register(MockType.NotRegister, "2")

        assert callbacks[MockType.Registered] == "1"
        assert callbacks[MockType.NotRegister] == "2"

    def test_get_unsupported_type(self):
        """测试获取不支持的值，返回默认值"""

        assert self.factory.get(MockType.NotRegister) == self.default_value

    def test_get_supported_type(self):
        """测试获取支持的值"""
        assert self.factory.get(MockType.Registered) == self.registered_value

    def test_get_none(self):
        """测试获取默认值"""
        assert self.factory.get() == self.default_value

    def test_must_get_unsupported_type(self):
        """测试获取不支持的值，抛出异常"""

        with pytest.raises(RuntimeError):
            self.factory.must_get(MockType.NotRegister)

    def test_must_get_supported_type(self):
        """测试获取支持的值"""
        assert self.factory.must_get(MockType.Registered) == self.registered_value

    def test_contains(self):
        """测试是否支持该类型"""

        assert MockType.Registered in self.factory
        assert MockType.NotRegister not in self.factory

    def test_clear(self):
        """测试清空注册的值"""

        assert MockType.Registered in self.factory

        self.factory.clear()

        assert MockType.Registered not in self.factory

    def test_getitem_unsupported_type(self):
        """测试获取不支持的值，抛出异常"""

        with pytest.raises(RuntimeError):
            self.factory[MockType.NotRegister]

    def test_getitem_supported_type(self):
        """测试获取支持的值"""

        assert self.factory[MockType.Registered] == self.registered_value

    def test_iter(self):
        """遍历已注册的类型"""

        registered_types = set(self.factory)

        assert MockType.Registered in registered_types
        assert MockType.NotRegister not in registered_types

    def test_len(self):
        """测试长度"""

        assert len(self.factory) == 1

        self.factory.register(MockType.NotRegister, "b")

        assert len(self.factory) == 2

    def test_keys(self):
        """测试注册的类型的键"""

        keys = self.factory.keys()
        assert MockType.Registered in keys
        assert MockType.NotRegister not in keys

    def test_registers(self):
        """测试注册的类型的值"""

        registers = self.factory.registers()
        assert self.registered_value in registers

    def test_items(self):
        """测试已注册的类型的键和值"""

        items = self.factory.items()
        assert (MockType.Registered, self.registered_value) in items


class TestSingletonFactory:
    """测试 SingletonFactory"""

    factory: SingletonFactory[MockType, str]

    @pytest.fixture(autouse=True)
    def setup(self, faker: ft.faker):
        self.default_value = faker.word()
        self.registered_value = faker.word()

        self.factory = SingletonFactory("testing", self.default_value)
        self.factory.register(MockType.Registered, self.registered_value)

    def test_factory_type(self):
        """测试 factory_type 默认值"""

        assert SingletonFactory.factory_type == "singleton"

    def test_call_without_args(self):
        """测试不传入参数时，返回默认值"""

        assert self.factory() == self.default_value

    def test_call_with_supported_type(self):
        """测试传入支持的类型时，返回对应值"""

        assert self.factory(MockType.Registered) == self.registered_value

    def test_call_with_unsupported_type(self):
        """测试传入不支持的类型时，返回默认值"""

        assert self.factory(MockType.NotRegister) == self.default_value


class TestSimpleFactory:
    """测试 SimpleFactory"""

    factory: SimpleFactory[MockType, str]

    @pytest.fixture(autouse=True)
    def setup(self, faker: ft.faker, mocker: ft.mocker):
        self.default_value = faker.word()
        self.registered_value = faker.word()

        self.factory = SimpleFactory("testing", mocker.MagicMock(return_value=self.default_value))
        self.factory.register(MockType.Registered, mocker.MagicMock(return_value=self.registered_value))

    def test_factory_type(self):
        """测试 factory_type 默认值"""

        assert self.factory.factory_type == "simple"
        assert self.registered_value != self.default_value

    def test_make_registered(self):
        """测试构造已注册的类型"""

        assert self.factory.make(MockType.Registered) == self.registered_value

    def test_make_unregistered(self):
        """测试构造未注册的类型，返回默认值"""

        assert self.factory.make(MockType.NotRegister) == self.default_value

    def test_must_make_registered(self):
        """测试构造已注册的类型，不抛出异常"""

        assert self.factory.must_make(MockType.Registered) == self.registered_value

    def test_must_make_unregistered(self):
        """测试构造未注册的类型，抛出异常"""

        with pytest.raises(RuntimeError):
            self.factory.must_make(MockType.NotRegister)

    def test_call_with_none(self):
        """测试不传入参数时，返回默认值"""

        assert self.factory() == self.default_value

    def test_getitem(self):
        """通过下标创建"""

        callback = self.factory[MockType.Registered]

        assert callback() == self.registered_value



import enum


class PromptRole(enum.Enum):
    """chat prompt 包含的角色"""

    ROLE = "role"  # 前端传入用于指定当前预设角色的提示词,需要转为system
    SYSTEM = "system"  # 前端传入的system需要被过滤
    TIME = "time"
    USER = "user"
    ASSISTANT = "assistant"
    AI = "ai"
    GUIDE = "guide"
    HIDDEN = "hidden"
    PAUSE = "pause"  # 暂停演绎,等待用户输入
    USER_IMAGE = "user-image"  # 用户带图片的输入


class ChatContentStatus(enum.Enum):
    LOADING = "loading"
    FAIL = "fail"
    SUCCESS = "success"


class IntentStatus(enum.Enum):
    QA_WITH_RETRIEVED_KNOWLEDGE_RESOURCES = "QA_WITH_RETRIEVED_KNOWLEDGE_RESOURCES"
    AGENT_FINISH_WITH_RESPONSE = "AGENT_FINISH_WITH_RESPONSE"
    DIRECTLY_RESPOND_BY_AGENT = "DIRECTLY_RESPOND_BY_AGENT"
    PROCESS_BY_AGENT = "PROCESS_BY_AGENT"


class Decision(enum.Enum):
    GENERAL_QA = "GENERAL_QA"
    PRIVATE_QA = "PRIVATE_QA"
    QUERY_CLARIFICATION = "QUERY_CLARIFICATION"


class FineGrainedScoreType(enum.Enum):
    LLM = "LLM"
    EXCLUSIVE_SIMILARITY_MODEL = "EXCLUSIVE_SIMILARITY_MODEL"
    EMBEDDING = "EMBEDDING"


class IndependentQueryMode(enum.Enum):
    INIT = "INIT"  # 直接使用原始的，不进行重写
    REWRITE = "REWRITE"  # 根据 chat history 对 query 进行重写
    SUM_AND_CONCATE = "SUM_AND_CONCATE"  # 对 chat history 进行总结，并跟 query 进行拼接


class KnowledgeBaseQueryFunction(enum.Enum):
    """检索方式"""

    SEMANTIC = "semantic"  # 语义检索
    MIXED = "mixed"  # 混合检索
    SQL = "sql"  # SQL检索


class IntentCategory(enum.Enum):
    KNOWLEDGE_BASE = "knowledge base"
    KNOWLEDGE_ITEM = "knowledge item"
    TOOL = "tool"


class StreamEventType(enum.Enum):
    NO = ""  # 不会展示这个
    LOADING = "loading"
    TEXT = "text"
    DONE = "done"
    ERROR = "error"
    REFERENCE_DOC = "reference_doc"
    THINK = "think"


class CredentialType(enum.Enum):
    NULL = "null"
    BLUEAPPS = "blueapps"  # apigw
    CUSTOM = "custom"  # proxy


class ContextType(enum.Enum):
    PRIVATE = "private"
    QA_RESPONSE = "qa_response"
    BOTH = "both"


class AgentBuildType(enum.Enum):
    """Agent构建类型"""

    SESSION = "session"
    DIRECT = "direct"


class AgentType(enum.Enum):
    """Agent类型"""

    CHAT = "chat"
    TASK = "task"



import logging
from abc import ABC, abstractmethod

from aidev_agent.utils.renderer import SafeJinjaEnvironment

logger = logging.getLogger("agent_command")

"""
新版快捷指令前后端协议
{
    "session_code": session_code,
    "role": "user",
    "content": "xxxx",
    "property": {
        "extra": {
            "cite": "待翻译文本: 你好, 语言: english",
            "command": "translate",
            "context": [
                {
                    "content": "你好",
                    "context_type": "textarea",
                    "__label": "待翻译文本",
                    "__key": "content",
                    "__value": "你好"
                },
                {
                    "language": "english",
                    "context_type": "select",
                    "__label": "语言",
                    "__key": "language",
                    "__value": "english"
                }
            ],
            "anchor_path_resources": {

            }
        }
    }
}
"""


class CommandHandler(ABC):
    """
    快捷指令处理器基类
    """

    agent_code = None  # 智能体代码
    command = None  # 指令名称

    def __init__(self):
        self.jinja_env = SafeJinjaEnvironment()

    @abstractmethod
    def get_template(self) -> str:
        """获取命令对应的提示词模板"""

    def extract_context_vars(self, context: list[dict]) -> dict[str, str]:
        """
        从上下文中提取模板变量（通用实现可被重写）
        """
        variables = {}
        for item in context:
            if "__key" in item and "__value" in item:
                variables[item["__key"]] = item["__value"]
        return variables

    def process_content(self, context: list[dict]) -> str:
        """
        处理内容（使用Jinja2模板和变量）
        """
        template = self.get_template()
        variables = self.extract_context_vars(context)

        return self.jinja_env.render(template, variables)


class TranslateCommandHandler(CommandHandler):
    """
    快捷指令: 翻译
    keyword: language 目标语言
    keyword: content 待翻译的内容
    """

    command = "translate"

    def get_template(self) -> str:
        return """
        请将以下内容翻译为{{ language }}:
        {{ content }}
        翻译要求: 确保翻译准确无误，无需冗余回答内容
        """

    def extract_context_vars(self, context: list[dict]) -> dict[str, str]:
        variables = super().extract_context_vars(context)
        # 特殊处理：确保必须有content变量
        if "content" not in variables:
            raise ValueError("Translation requires 'content' in context")
        return variables


class ExplanationCommandHandler(CommandHandler):
    """
    快捷指令: 解释
    keyword: content 待解释的内容
    """

    command = "explanation"

    def get_template(self) -> str:
        return """
        请解释以下内容{{ content }}
        解释要求: 确保解释准确无误，无需冗余回答内容
        """


class CommonCommandHandler(CommandHandler):
    def __init__(self, command: str, template: str):
        super().__init__()
        self._template = template
        self.command = command

    def get_template(self) -> str:
        return self._template


class CommonCommandHandlerBuilder:
    @classmethod
    def build(cls, command_id, command_template) -> CommonCommandHandler:
        return CommonCommandHandler(command_id, command_template)


class CommandProcessor:
    _handlers: dict[str, type[CommandHandler]] = {}

    @classmethod
    def register_handler(cls, command: str, handler: type[CommandHandler]):
        """
        注册快捷指令
        """
        cls._handlers[command] = handler

    def process_command(self, command_data: dict) -> str:
        """
        处理快捷指令
        """
        if not (command := command_data.get("command")):
            logger.warning("CommandProcessor: No command found in data->[%s]", command_data)
            raise ValueError("No command found in data")

        if (handler_class := self._handlers.get(command)) is None:
            logger.warning("CommandProcessor: No handler registered for command->[%s]", command)
            raise ValueError(f"No handler registered for command: {command}")

        try:
            logger.info("CommandProcessor: Processing command->[%s]", command)
            return handler_class().process_content(command_data.get("context", []))
        except ValueError as e:
            logger.warning("CommandProcessor: Command processing failed->[%s]", str(e))
            raise e


# 注册处理器 exsample
# CommandProcessor.register_handler("translate", TranslateCommandHandler)
# CommandProcessor.register_handler("explanation", ExplanationCommandHandler)



import os
from typing import Any, List, Literal, Tuple

from pydantic import AliasChoices, BaseModel, Field, model_validator

from aidev_agent.enums import FineGrainedScoreType, IndependentQueryMode, KnowledgeBaseQueryFunction


class ExecuteKwargs(BaseModel):
    stream: bool = False
    stream_timeout: int = 30
    passthrough_input: bool = False
    run_agent: bool = False
    # 新增参数
    caller_bk_app_code: str | None = Field(default=None, description="调用者BK应用ID")
    caller_bk_biz_env: str | None = Field(default=None, description="调用者BK业务环境")
    caller_bk_biz_id: int | None = Field(default=None, description="调用者BK业务ID")
    caller_executor: str | None = Field(default=None, description="调用人")
    caller_order_type: str | None = Field(default=None, description="调用AI工单类型")
    caller_trace_id: str | None = Field(default=None, description="调用链ID")


class SessionTool(BaseModel):
    tool_id: int
    tool_code: str
    icon: str
    tool_name: str = Field(validation_alias=AliasChoices("tool_name", "tool_cn_name"))
    description: str
    is_sensitive: bool
    status: Literal["ready", "deleted"] = "ready"
    property: dict = Field(default_factory=dict)

    @classmethod
    def get_model_fields_list_without_default_values(cls) -> list[str]:
        field_list = []
        for name, field_info in cls.model_fields.items():
            if field_info.default:
                continue
            field_list.append(name)
        return field_list


class SessionContentExtra(BaseModel):
    """会话内容的一些额外属性"""

    tools: list[SessionTool] = Field(default_factory=list)
    anchor_path_resources: dict = Field(default_factory=dict)
    context: list[dict] | None = None
    command: str | None = None
    rendered_content: str | None = None


class SessionContentProperty(BaseModel):
    """会话内容的一些额外属性"""

    extra: SessionContentExtra | None = None


class ChatPrompt(BaseModel):
    id: int | None = None
    role: str
    content: str
    extra: SessionContentExtra | None = None

    @model_validator(mode="before")
    def validate_content_with_rendered(cls, values: Any) -> Any:
        extra = values.get("extra")
        if extra:
            if isinstance(extra, dict):
                rendered_content = extra.get("rendered_content")
                if rendered_content:
                    values["content"] = rendered_content
            elif hasattr(extra, "rendered_content") and extra.rendered_content:
                values["content"] = extra.rendered_content
        return values


class IntentRecognition(BaseModel):
    intent_recognition_knowledge: list[dict] | None = Field(default=None, description=("意图识别知识"))
    intent_recognition_topk: float | None = Field(default=None, description=("意图识别topk值"))
    intent_recognition_llm: str | None = Field(
        default=os.getenv("INTENT_RECOGNITION_LLM"), description="意图识别使用的LLM"
    )
    intent_recognition_fine_grained_score_type: FineGrainedScoreType = Field(
        default=FineGrainedScoreType(os.getenv("INTENT_FINE_GRAINED_SCORE_TYPE", "LLM")), description=("相关性判断模型")
    )
    intent_recognition_reject_threshold: Tuple[float, float] = Field(
        default=(
            float(os.getenv("INTENT_REJECT_THRESHOLD_MIN", "0.001")),
            float(os.getenv("INTENT_REJECT_THRESHOLD_MAX", "0.6")),
        ),
        description=("相关性阈值"),
    )
    enable_logging: bool = Field(
        default=os.getenv("ENABLE_LOGGING", "true").lower() == "true", description="是否启用日志记录"
    )
    intent_recognition_llm_code: str | None = Field(
        default=os.getenv("INTENT_RECOGNITION_LLM_CODE"), description=("约定的意图识别 code，用于快速单跳")
    )
    with_index_specific_search_init: bool = Field(
        default=os.getenv("WITH_INDEX_SPECIFIC_SEARCH_INIT", "true").lower() == "true",
        description="是否使用初始查询进行 index specific 召回",
    )
    with_index_specific_search_translation: bool = Field(
        default=os.getenv("WITH_INDEX_SPECIFIC_SEARCH_TRANSLATION", "false").lower() == "true",
        description="是否使用翻译后的查询进行 index specific 召回",
    )
    with_index_specific_search_keywords: bool = Field(
        default=os.getenv("WITH_INDEX_SPECIFIC_SEARCH_KEYWORDS", "false").lower() == "true",
        description="是否使用提取的关键词进行 index specific 召回",
    )
    tool_output_compress_thrd: int = Field(
        default=int(os.getenv("TOOL_OUTPUT_COMPRESS_THRD", "5000")), description=("工具输出压缩阈值")
    )
    agent_type: str | None = Field(default=os.getenv("AGENT_TYPE"), description=("agent类"))
    max_tool_output_len: int = Field(
        default=int(os.getenv("MAX_TOOL_OUTPUT_LEN", "500")), description=("工具调用结果展示的最大长度")
    )
    max_cache_length: int = Field(default=int(os.getenv("MAX_CACHE_LENGTH", "50")), description=("缓存的最大长度"))
    max_iterations: int = Field(default=int(os.getenv("MAX_ITERATIONS", "50")), description=("最大迭代次数"))
    non_thinking_llm: str = Field(default=os.getenv("NON_THINKING_LLM", "hunyuan"), description=("非深度思考模型"))


class KnowledgebaseSettings(BaseModel):
    knowledge_bases: list[dict] = Field(default_factory=list, description=("关联知识库,有可能没有关联"))
    knowledge_items: list[dict] = Field(default_factory=list, description=("关联知识,可能没有关联"))
    qa_response_kb_ids: list[int] = Field(default_factory=list, description=("历史反馈问答知识库id,可能不存在"))
    qa_response_knowledge_bases: list[dict] = Field(default_factory=list, description=("历史反馈问答知识库,可能不存在"))
    retriever_code: str | None = Field(default=os.getenv("RETRIEVER_CODE"), max_length=255, description=("检索器ID"))
    query_function: KnowledgeBaseQueryFunction = Field(
        default=KnowledgeBaseQueryFunction(os.getenv("QUERY_FUNCTION", "semantic")), description=("查询方式")
    )
    document_fragment_count: int = Field(
        default=int(os.getenv("DOCUMENT_FRAGMENT_COUNT", "0")), description=("文档片段数")
    )
    knowledge_resource_fine_grained_score_type: FineGrainedScoreType = Field(
        default=FineGrainedScoreType(os.getenv("KNOWLEDGE_FINE_GRAINED_SCORE_TYPE", "LLM")),
        description=("相关性判断模型"),
    )
    knowledge_resource_reject_threshold: Tuple[float, float] = Field(
        default=(
            float(os.getenv("KNOWLEDGE_REJECT_THRESHOLD_MIN", "0.001")),
            float(os.getenv("KNOWLEDGE_REJECT_THRESHOLD_MAX", "0.1")),
        ),
        description=("相关性阈值"),
    )
    knowledge_resource_rough_recall_topk: int = Field(
        default=int(os.getenv("KNOWLEDGE_ROUGH_RECALL_TOPK", "10")),
        description=("知识类资源粗召 topk 值"),
        alias="topk",
    )
    independent_query_mode: IndependentQueryMode = Field(
        default=IndependentQueryMode(os.getenv("INDEPENDENT_QUERY_MODE", "SUM_AND_CONCATE")), description=("预处理逻辑")
    )
    polish: bool = Field(
        default=os.getenv("POLISH", "true").lower() == "true", description=("是否返回检索原始文档内容")
    )
    raw: bool = Field(default=os.getenv("RAW", "false").lower() == "true", description=("是否返回检索大模型总结内容"))
    knowledge_template_id: int | None = Field(
        default=int(os.getenv("KNOWLEDGE_TEMPLATE_ID", "0")) if os.getenv("AGENT_KNOWLEDGE_TEMPLATE_ID") else None,
        description=("检索内容返回模板ID"),
    )
    is_response_when_no_knowledgebase_match: bool = Field(
        default=os.getenv("IS_RESPONSE_WHEN_NO_KNOWLEDGEBASE_MATCH", "true").lower() == "true",
        description=("未命中知识库时根据通识回答"),
    )
    rejection_message: str = Field(
        default=os.getenv("REJECTION_MESSAGE", "无法根据当前文档回答当前问题。请更换问题。"),
        max_length=1024,
        description=("拒答文案"),
    )
    enable_parallel_tool_calls: bool = Field(
        default=os.getenv("ENABLE_PARALLEL_TOOL_CALLS", "true").lower() == "true",
        description=("StructuredChatCommonQAAgent调用多个工具时是否使用并行调用"),
    )
    with_scalar_data: bool = Field(
        default=os.getenv("WITH_SCALAR_DATA", "false").lower() == "true",
        description="是否使用标量索引进行结构化数据召回",
    )
    use_independent_query_in_translation: bool = Field(
        default=os.getenv("USE_INDEPENDENT_QUERY_IN_TRANSLATION", "false").lower() == "true",
        description=("翻译查询时是否使用独立查询(而非原始查询)作为输入源"),
    )
    use_translated_query_in_scores: bool = Field(
        default=os.getenv("USE_TRANSLATED_QUERY_IN_SCORES", "true").lower() == "true",
        description="计算相关性分数时是否使用翻译后的查询(而非原始查询)",
    )
    use_independent_query_in_scores: bool = Field(
        default=os.getenv("USE_INDEPENDENT_QUERY_IN_SCORES", "true").lower() == "true",
        description="计算相关性分数时是否使用独立查询(而非原始查询)",
    )
    with_query_cls: bool = Field(
        default=os.getenv("WITH_QUERY_CLS", "true").lower() == "true",
        description="是否进行意图切换检测。NOTE: 目前仅在非 merge_query_cls_with_resp_or_rewrite 的情况下才生效",
    )
    force_process_by_agent: bool = Field(
        default=os.getenv("FORCE_PROCESS_BY_AGENT", "false").lower() == "true",
        description=("是否强制进入 IntentStatus.PROCESS_BY_AGENT 的 status。用于 AIDEV 产品页面召回测试"),
    )
    role_prompt: str | None = Field(
        default=os.getenv("ROLE_PROMPT"),
        description=(
            "用户在 aidev 页面上创建 agent 时填写的 prompt。旧主站逻辑会将其与 prefix 拼接后作为整体外层 agent 的 system prompt。"
        ),
    )
    assets_list: str | None = Field(default=os.getenv("ASSETS_LIST"), description=("assets_list参数"))
    with_structured_data: bool = Field(
        default=os.getenv("WITH_STRUCTURED_DATA", "false").lower() == "true",
        description=("用户勾选的知识中是否带结构化数据。NOTE: 目前该值为 True 时才会进行 nature 方式的知识召回"),
    )
    with_index_specific_search: bool = Field(
        default=os.getenv("WITH_INDEX_SPECIFIC_SEARCH", "true").lower() == "true",
        description="是否使用基于 embedding 模型的 index specific 召回",
    )
    with_es_search_query: bool = Field(
        default=os.getenv("WITH_ES_SEARCH_QUERY", "false").lower() == "true",
        description="是否使用原始 query 在 ES 上进行召回",
    )
    with_es_search_keywords: bool = Field(
        default=os.getenv("WITH_ES_SEARCH_KEYWORDS", "false").lower() == "true",
        description="是否使用 query 提取的关键词 在 ES 上进行召回",
    )
    with_rrf: bool = Field(
        default=os.getenv("WITH_RRF", "true").lower() == "true",
        description="是否使用 weighted reciprocal rank fusion 对多路召回的结果进行融合。",
    )
    self_query_threshold_top_n: int = Field(
        default=int(os.getenv("SELF_QUERY_THRESHOLD_TOP_N", "0")),
        description="""在第 1 步意图识别判断用户 query 是否涉及结构化数据的时候，
            使用全量检索结果的 top_n 中是否包含结构化数据来判断。
            NOTE: 当前先将其值置为0，即不开启 self query 分支。
            TODO: 后续去除 nature 分支后，即可将其值置为 5 等，开启 self query 分支。""",
    )
    tool_resource_rough_recall_topk: int = Field(
        default=int(os.getenv("TOOL_RESOURCE_ROUGH_RECALL_TOPK", "10")), description="工具类资源粗召 topk 值"
    )
    tool_resource_reject_threshold: Tuple[float, float] = Field(
        default=(
            float(os.getenv("TOOL_RESOURCE_REJECT_THRESHOLD_MIN", "0.25")),
            float(os.getenv("TOOL_RESOURCE_REJECT_THRESHOLD_MAX", "0.75")),
        ),
        description="工具类资源拒答和直接回答的阈值",
    )
    tool_resource_fine_grained_score_type: FineGrainedScoreType = Field(
        default=FineGrainedScoreType(os.getenv("TOOL_RESOURCE_FINE_GRAINED_SCORE_TYPE", "EXCLUSIVE_SIMILARITY_MODEL")),
        description="工具类资源进行细粒度的相似度计算的模型类型",
    )
    tool_count_threshold: int = Field(
        default=int(os.getenv("TOOL_COUNT_THRESHOLD", "10000")),
        description="当能够获取的工具的个数大于该阈值时，才开启工具类资源的粗召 + 精排，否则每次调用兜底LLM agent时都附上所有工具类资源",
    )
    gen_pseudo_tool_resource_desc: bool = Field(
        default=os.getenv("GEN_PSEUDO_TOOL_RESOURCE_DESC", "true").lower() == "true",
        description="在进行工具类资源的粗召时，是否进行伪工具类资源描述的生成",
    )
    retrieved_knowledge_resources: list = Field(default_factory=list, description="用户自带的历史检索的上下文")
    merge_query_cls_with_resp_or_rewrite: bool = Field(
        default=os.getenv("MERGE_QUERY_CLS_WITH_RESP_OR_REWRITE", "false").lower() == "true",
        description="是否将意图切换检测和 query 重写/直接答复合并在一次LLM调用中",
    )
    tool_resource_base_ids: List[int] = Field(
        default_factory=list,
        description=("工具类资源 base ID 列表。NOTE: 目前工具类资源统一放 base ID 中不放 item ID 中"),
    )
    token_limit_margin: int = Field(
        default=int(os.getenv("TOKEN_LIMIT_MARGIN", "100")), description=("上下文最大Token限制边界")
    )
    llm_token_limit: int = Field(default=int(os.getenv("LLM_TOKEN_LIMIT", "28000")), description=("LLM最大Token限制"))


class AgentOptions(BaseModel):
    # agent 执行选项
    intent_recognition_options: IntentRecognition = Field(default_factory=IntentRecognition, description="意图识别选项")
    knowledge_query_options: KnowledgebaseSettings = Field(
        default_factory=KnowledgebaseSettings, description="知识库查询选项"
    )



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import os
import time
from abc import abstractmethod
from concurrent.futures import Future
from concurrent.futures.thread import ThreadPoolExecutor
from pathlib import PosixPath
from typing import Dict, Optional

import Pyro4
from environs import Env
from Pyro4 import Daemon
from Pyro4.core import Proxy
from wrapt import synchronized

from . import service
from .service import dedicated_service_daemon, get_service_proxy


class AlreadySetupError(Exception):
    pass


thread_pool = ThreadPoolExecutor()


class ServiceRegistry:
    """远程服务注册"""

    _configured = False
    env = Env()

    def __init__(self, auto_configure: bool = True) -> None:
        self.current_daemon: Optional[Daemon] = None
        self.current_daemon_future: Optional[Future] = None
        if auto_configure:
            self.configure()

    @synchronized
    @classmethod
    def configure(
        cls,
        host: Optional[str] = None,
        ns_host: Optional[str] = None,
        ns_port: Optional[int] = None,
        contents_dir: Optional[str] = None,
    ) -> None:
        """配置"""
        if not cls._configured:
            if cls.env.str("BCS_RANDHOSTPORT_HOSTIP", None):
                Pyro4.config.NATHOST = cls.env.str("BCS_RANDHOSTPORT_HOSTIP")
                host = host or "0.0.0.0"
            Pyro4.config.HOST = (
                host
                or cls.env.str("DISPATCHING_DEDICATED_SERVICE_HOST", None)
                or cls.env.str("DEDICATED_SERVICE_HOST", "localhost")
                or cls.env.str("MODEL_SERVICE_HOST", "localhost")
            )
            Pyro4.config.NS_HOST = (
                ns_host
                or cls.env.str("DISPATCHING_DEDICATED_SERVICE_NS_HOST", None)
                or cls.env.str("DEDICATED_SERVICE_NS_HOST", "")
                or cls.env.str("MODEL_MANAGEMENT_NS_HOST", "")
            )
            Pyro4.config.NS_PORT = (
                ns_port
                or cls.env.int("DISPATCHING_DEDICATED_SERVICE_NS_PORT", None)
                or cls.env.int("DEDICATED_SERVICE_NS_PORT", None)
                or cls.env.int("MODEL_MANAGEMENT_NS_PORT", None)
            )
            env_contents_dir = cls.env.str("DISPATCHING_DEDICATED_SERVICE_CONTENTS_DIR", None) or cls.env.str(
                "DEDICATED_SERVICE_CONTENTS_DIR", None
            )
            contents_dir = str(
                PosixPath(contents_dir)
                if contents_dir
                else (PosixPath(env_contents_dir) if env_contents_dir else PosixPath.cwd())
            )
            Pyro4.config.SSL_CACERTS = Pyro4.config.SSL_SERVERCERT = Pyro4.config.SSL_CLIENTCERT = (
                f"{contents_dir}/certs/cert.pem"
            )
            Pyro4.config.SSL_SERVERKEY = Pyro4.config.SSL_CLIENTKEY = f"{contents_dir}/certs/key.pem"
            encryption = (
                cls.env.str("MODEL_SERVICE_ENCRYPTION", None)
                or cls.env.str("DISPATCHING_DEDICATED_SERVICE_ENCRYPTION", None)
                or cls.env.str("DEDICATED_SERVICE_ENCRYPTION", "protocol_fernet")
            )
            if encryption != "ssl":
                Pyro4.config.SSL = False
                Pyro4.config.SSL_REQUIRECLIENTCERT = False
                if encryption in ("protocol_aes", "protocol_sm4", "protocol_fernet"):
                    service.CUSTOM_ENCRYPTION = encryption

    @synchronized
    def register(self, object_qualified_name_map: Dict, main_service_name: str, port=None):
        """注册服务对象"""
        if self.current_daemon:
            raise AlreadySetupError("服务已注册")
        if self.env.str("BCS_RANDHOSTPORT_HOSTIP", None):
            Pyro4.config.NATPORT = self.env.int("BCS_RANDHOSTPORT_FOR_CONTAINER_PORT_7000")
            port = port or 7000
        port = (
            port or self.env.int("DISPATCHING_DEDICATED_SERVICE_PORT", None) or self.env.int("DEDICATED_SERVICE_PORT")
        )
        try:
            self.current_daemon = dedicated_service_daemon(
                object_qualified_name_map, main_service_name, port=port, looping=False
            )
            self.current_daemon_future = thread_pool.submit(self.current_daemon.requestLoop)
            for i in range(10):
                if self.current_daemon_future.running():
                    break
                time.sleep(1)
            else:
                raise TimeoutError(f"{main_service_name}{object_qualified_name_map}注册未成功")
        except Exception:
            self.unregister()
            raise

    @synchronized
    def unregister(self):
        """反注册服务对象"""
        if self.current_daemon and self.current_daemon_future:
            if self.current_daemon:
                self.current_daemon.shutdown()
            for i in range(60):
                if self.current_daemon_future.done():
                    break
                time.sleep(1)
            else:
                raise TimeoutError("反注册未成功")
            self.current_daemon = None
            self.current_daemon_future = None

    def get(self, object_qualified_name: str, cached: bool) -> Proxy:
        """获取服务对象"""
        obj = get_service_proxy(object_qualified_name, cached)
        return obj


class LLMRegistry(ServiceRegistry):
    @synchronized
    def register(self, object_qualified_name_map: Dict, main_service_name: str, port=None):
        if not main_service_name.startswith("bkaidev_llm"):
            raise Exception(f"不合规的MainDedicatedServiceName: {main_service_name}")
        return super().register(
            object_qualified_name_map=object_qualified_name_map, main_service_name=main_service_name, port=port
        )


class RegistryPluginMixIn:
    """服务注册快捷工具"""

    # 服务注册的前缀
    SERVICE_PREFIX = os.environ.get("LLM_SERVICE_PREFIX", "bkaidev_llm")

    @property
    @abstractmethod
    def service_name(self):
        """服务名称"""

    def get_registered_object(self, cached: bool = False, service_name: None = None) -> Proxy:
        """获取已注册的object"""
        from .registry import LLMRegistry

        service_name = service_name or self.service_name
        return LLMRegistry().get(f"{self.SERVICE_PREFIX}.{service_name}", cached=cached)



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import asyncio
import atexit
import threading

# Thread-local storage for event loops
_thread_local = threading.local()


def get_event_loop():
    """
    Get the current event loop for this thread, create one if it doesn't exist.

    Returns:
        asyncio.AbstractEventLoop: The current event loop for this thread.
    """
    # Check if we have a cached loop for this thread
    if hasattr(_thread_local, "loop") and _thread_local.loop is not None and not _thread_local.loop.is_closed():
        return _thread_local.loop

    try:
        # Try to get the running loop first
        running_loop = asyncio.get_running_loop()
        _thread_local.loop = running_loop
        return running_loop
    except RuntimeError:
        # No running loop, try to get the event loop for this thread
        try:
            current_loop = asyncio.get_event_loop()
            # Check if this loop is running
            if current_loop.is_running():
                # If the loop is running, we can't use run_until_complete on it
                # Create a new loop for this thread
                new_loop = asyncio.new_event_loop()
                asyncio.set_event_loop(new_loop)
                _thread_local.loop = new_loop
                # Register cleanup function for this thread's loop
                atexit.register(_cleanup_thread_loop, new_loop)
                return new_loop
            else:
                _thread_local.loop = current_loop
                return current_loop
        except RuntimeError:
            # No event loop exists, create a new one
            new_loop = asyncio.new_event_loop()
            asyncio.set_event_loop(new_loop)
            _thread_local.loop = new_loop
            # Register cleanup function for this thread's loop
            atexit.register(_cleanup_thread_loop, new_loop)
            return new_loop


def _cleanup_thread_loop(loop):
    """Cleanup function to properly close a thread's event loop on exit."""
    if loop is not None and not loop.is_closed():
        try:
            # Cancel all pending tasks
            pending = asyncio.all_tasks(loop)
            for task in pending:
                task.cancel()

            # Run until all tasks are cancelled if the loop is not running
            if pending and not loop.is_running():
                loop.run_until_complete(asyncio.gather(*pending, return_exceptions=True))

            # Close the loop
            loop.close()
        except Exception:
            # Ignore exceptions during cleanup
            pass



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import json
from unittest.mock import Mock

import pytest
from aidev_agent.core.extend.agent.qa import (
    CommonQAStreamingMixIn,
    StructuredChatCommonQAAgent,
    ToolCallingCommonQAAgent,
)
from aidev_agent.enums import StreamEventType
from aidev_agent.utils import Empty
from langchain_core.messages import AIMessageChunk


class MockAgent:
    """模拟 Agent 用于测试"""

    def stream_events(self, input_, config=None, version="v2", timeout=2):
        """模拟 stream_events 方法"""


class TestCommonQAStreamingMixIn:
    """测试 CommonQAStreamingMixIn 的 stream_standard_event 方法"""

    @pytest.fixture
    def mock_tool_calling_agent(self):
        """创建一个 mock 的 ToolCallingCommonQAAgent 实例"""
        agent = Mock(spec=ToolCallingCommonQAAgent)
        agent.LOADING_AGENT_MESSAGE = "正在思考..."
        agent.think_symbols = ["<think>\n", "\n</think>\n"]
        agent.final_answer_prefixes = ["Final Answer:", "最终答案："]
        agent.final_answer_suffixes = ["\n```\n", "\n```"]
        agent.end_content = "<｜end▁of▁sentence｜>"
        agent.llm = Mock()
        agent._yield_ret = CommonQAStreamingMixIn._yield_ret.__get__(agent, type(agent))
        agent.check_and_append = CommonQAStreamingMixIn.check_and_append.__get__(agent, type(agent))
        agent.common_filter = CommonQAStreamingMixIn.common_filter.__get__(agent, type(agent))
        agent.cache_filter = CommonQAStreamingMixIn.cache_filter.__get__(agent, type(agent))
        agent.stream_standard_event = CommonQAStreamingMixIn.stream_standard_event.__get__(agent, type(agent))
        return agent

    @pytest.fixture
    def mock_structured_chat_agent(self):
        """创建一个 mock 的 StructuredChatCommonQAAgent 实例"""
        agent = Mock(spec=StructuredChatCommonQAAgent)
        agent.LOADING_AGENT_MESSAGE = "正在思考..."
        agent.think_symbols = ["<think>\n", "\n</think>\n"]
        agent.final_answer_prefixes = ["Final Answer:", "最终答案："]
        agent.final_answer_suffixes = ["\n```\n", "\n```"]
        agent.end_content = "<｜end▁of▁sentence｜>"
        agent.llm = Mock()
        agent._yield_ret = CommonQAStreamingMixIn._yield_ret.__get__(agent, type(agent))
        agent.check_and_append = CommonQAStreamingMixIn.check_and_append.__get__(agent, type(agent))
        agent.common_filter = CommonQAStreamingMixIn.common_filter.__get__(agent, type(agent))
        agent.cache_filter = CommonQAStreamingMixIn.cache_filter.__get__(agent, type(agent))
        agent.stream_standard_event = CommonQAStreamingMixIn.stream_standard_event.__get__(agent, type(agent))
        return agent

    def test_loading_event_type(self, mock_tool_calling_agent):
        """测试 LOADING 事件类型（通过 Empty 触发）"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            yield Empty
            yield Empty
            # 添加一个正常的内容以确保流程完整
            chunk = AIMessageChunk(content="完成")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 验证产生了 LOADING 消息（在 data: 格式的字符串中）
        loading_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("content") == "正在思考...":
                    loading_events.append(data)

        assert len(loading_events) >= 1

    def test_text_event_type_tool_calling(self, mock_tool_calling_agent):
        """测试 TEXT 事件类型（ToolCallingCommonQAAgent）"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟普通文本内容
            chunk = AIMessageChunk(content="这是一个测试回答")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        text_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.TEXT.value:
                    text_events.append(data)

        assert len(text_events) > 0
        assert any("测试回答" in e.get("content", "") for e in text_events)

    def test_think_event_type_tool_calling(self, mock_tool_calling_agent):
        """测试 THINK 事件类型（ToolCallingCommonQAAgent - reasoning_content）"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟 reasoning_content
            chunk = AIMessageChunk(content="", additional_kwargs={"reasoning_content": "让我思考一下..."})
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }
            # 模拟最终答案
            chunk2 = AIMessageChunk(content="最终答案")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk2},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        think_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)

        assert len(think_events) > 0
        assert any("思考" in e.get("content", "") for e in think_events)

    def test_think_event_type_tool_call(self, mock_tool_calling_agent):
        """测试 THINK 事件类型（ToolCallingCommonQAAgent - tool_call）"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟工具调用 - 工具名称
            chunk1 = AIMessageChunk(content="", tool_call_chunks=[{"name": "weather_query"}])
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk1},
                "run_id": "test_run_1",
            }
            # 模拟工具调用 - 参数
            chunk2 = AIMessageChunk(content="", tool_call_chunks=[{"args": '{"city": "深圳"}'}])
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk2},
                "run_id": "test_run_1",
            }
            # 模拟工具调用结束
            yield {
                "event": "on_tool_end",
                "data": {"output": "深圳今天晴天，温度25度"},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        think_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)

        assert len(think_events) > 0
        # 验证包含工具调用相关内容
        all_content = "".join(e.get("content", "") for e in think_events)
        assert "weather_query" in all_content or "Agent Action" in all_content

    def test_reference_doc_event_type(self, mock_tool_calling_agent):
        """测试 REFERENCE_DOC 事件类型"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟 reference_doc 自定义事件
            yield {
                "event": "on_custom_event",
                "data": {
                    "reference_doc": [
                        {"file_path": "/path/to/doc1.txt", "score": 0.95},
                        {"file_path": "/path/to/doc2.txt", "score": 0.88},
                    ]
                },
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        reference_doc_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.REFERENCE_DOC.value:
                    reference_doc_events.append(data)

        assert len(reference_doc_events) > 0
        assert "documents" in reference_doc_events[0]
        assert len(reference_doc_events[0]["documents"]) == 2

    def test_done_event_type(self, mock_tool_calling_agent):
        """测试 DONE 事件类型"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            chunk = AIMessageChunk(content="测试完成")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 验证最后一个事件是 DONE
        assert results[-1] == "data: [DONE]\n\n"

        # 验证倒数第二个事件包含 DONE event type
        done_events = []
        for r in results[:-1]:
            if r.startswith("data: "):
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.DONE.value:
                    done_events.append(data)

        assert len(done_events) > 0

    def test_error_event_type(self, mock_tool_calling_agent):
        """测试 ERROR 事件类型（通过异常触发）"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟抛出异常
            raise RuntimeError("测试错误")

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 验证产生了错误事件
        error_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.ERROR.value:
                    error_events.append(data)

        assert len(error_events) > 0

    def test_custom_event_compress_log(self, mock_tool_calling_agent):
        """测试自定义事件 - compress_log"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            yield {
                "event": "on_custom_event",
                "data": {"compress_log": "Token 超限，尝试压缩..."},
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        think_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)

        assert len(think_events) > 0
        assert any("压缩" in e.get("content", "") for e in think_events)

    def test_custom_event_intent_recognition(self, mock_tool_calling_agent):
        """测试自定义事件 - intent_recognition_result"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            yield {
                "event": "on_custom_event",
                "data": {"intent_recognition_result": "识别到用户意图：查询天气"},
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        think_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)

        assert len(think_events) > 0
        assert any("意图" in e.get("content", "") for e in think_events)

    def test_custom_event_custom_return_chunk(self, mock_tool_calling_agent):
        """测试自定义事件 - custom_return_chunk"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            yield {
                "event": "on_custom_event",
                "data": {"custom_return_chunk": "自定义返回内容"},
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        text_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.TEXT.value:
                    text_events.append(data)

        assert len(text_events) > 0
        assert any("自定义返回内容" in e.get("content", "") for e in text_events)

    def test_custom_event_custom_agent_finish(self, mock_tool_calling_agent):
        """测试自定义事件 - custom_agent_finish"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            yield {
                "event": "on_custom_event",
                "data": {"custom_agent_finish": "Agent 执行完成"},
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        text_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.TEXT.value:
                    text_events.append(data)

        assert len(text_events) > 0
        assert any("完成" in e.get("content", "") for e in text_events)

    def test_front_end_display_control(self, mock_tool_calling_agent):
        """测试 front_end_display 控制"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 关闭前端显示
            yield {
                "event": "on_custom_event",
                "data": {"front_end_display": False},
            }
            # 这条消息不应该被显示
            chunk = AIMessageChunk(content="不应该显示的内容")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }
            # 重新开启前端显示
            yield {
                "event": "on_custom_event",
                "data": {"front_end_display": True},
            }
            # 这条消息应该被显示
            chunk2 = AIMessageChunk(content="应该显示的内容")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk2},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        all_content = ""
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.TEXT.value:
                    all_content += data.get("content", "")

        assert "不应该显示的内容" not in all_content
        assert "应该显示的内容" in all_content

    def test_structured_chat_final_answer(self, mock_structured_chat_agent):
        """测试 StructuredChatCommonQAAgent 的 Final Answer 处理"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟思考过程
            chunk1 = AIMessageChunk(content="让我思考一下...")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk1},
                "run_id": "test_run_1",
            }
            # 模拟 Final Answer 前缀
            chunk2 = AIMessageChunk(content="Final Answer:")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk2},
                "run_id": "test_run_1",
            }
            # 模拟最终答案
            chunk3 = AIMessageChunk(content="这是最终答案")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk3},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_structured_chat_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        think_events = []
        text_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)
                elif data.get("event") == StreamEventType.TEXT.value:
                    text_events.append(data)

        # 验证思考过程和最终答案分离
        assert len(think_events) > 0
        assert len(text_events) > 0

    def test_tool_output_error_handling(self, mock_tool_calling_agent):
        """测试工具输出错误处理"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟工具调用错误
            yield {
                "event": "on_tool_end",
                "data": {"output": "invalid_tool is not a valid tool, try one of [tool1, tool2]"},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果
        think_events = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)

        assert len(think_events) > 0
        all_content = "".join(e.get("content", "") for e in think_events)
        assert "工具调用失败" in all_content or "invalid_tool" in all_content

    def test_skip_thought_parameter(self, mock_tool_calling_agent):
        """测试 skip_thought 参数

        注意：skip_thought=True 时，会跳过所有 on_chat_model_stream 事件的处理，
        因此这个测试主要验证 skip_thought 参数的存在性和基本功能。
        """
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟 reasoning_content（应该被跳过）
            chunk = AIMessageChunk(content="", additional_kwargs={"reasoning_content": "思考内容"})
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }
            # 模拟文本内容（也会被跳过，因为 skip_thought=True）
            chunk2 = AIMessageChunk(content="文本内容")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk2},
                "run_id": "test_run_1",
            }
            # 模拟自定义事件（不会被跳过）
            yield {
                "event": "on_custom_event",
                "data": {"custom_return_chunk": "自定义内容"},
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        # 使用 skip_thought=True
        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_, skip_thought=True))

        # 解析结果
        think_events = []
        text_events = []
        done_events = []
        for r in results:
            if r == "data: [DONE]\n\n":
                continue
            if r.startswith("data: "):
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value:
                    think_events.append(data)
                elif data.get("event") == StreamEventType.TEXT.value:
                    text_events.append(data)
                elif data.get("event") == StreamEventType.DONE.value:
                    done_events.append(data)

        # 验证 on_chat_model_stream 的思考内容被跳过
        assert len(think_events) == 0
        # 验证自定义事件的文本内容被保留，或至少有 DONE 事件
        assert len(text_events) > 0 or len(done_events) > 0

    def test_elapsed_time_in_think_event(self, mock_tool_calling_agent):
        """测试 THINK 事件中的 elapsed_time"""
        mock_agent_e = MockAgent()

        def mock_stream_events(*args, **kwargs):
            # 模拟 reasoning_content
            chunk = AIMessageChunk(content="", additional_kwargs={"reasoning_content": "思考中..."})
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk},
                "run_id": "test_run_1",
            }
            # 模拟切换到文本内容
            chunk2 = AIMessageChunk(content="答案")
            yield {
                "event": "on_chat_model_stream",
                "data": {"chunk": chunk2},
                "run_id": "test_run_1",
            }

        mock_agent_e.stream_events = mock_stream_events

        cfg = {}
        input_ = {"input": "test"}

        results = list(mock_tool_calling_agent.stream_standard_event(mock_agent_e, cfg, input_))

        # 解析结果，查找带 elapsed_time 的 think event
        think_events_with_time = []
        for r in results:
            if r.startswith("data: ") and r != "data: [DONE]\n\n":
                data = json.loads(r[6:])
                if data.get("event") == StreamEventType.THINK.value and "elapsed_time" in data:
                    think_events_with_time.append(data)

        # 验证存在带 elapsed_time 的 think event
        assert len(think_events_with_time) > 0
        assert think_events_with_time[0]["elapsed_time"] >= 0



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import os
import signal
from typing import Any, Dict, List, Optional, Tuple, Union

from Pyro4.message import Message

if os.getenv("GEVENT_IN_DEDICATED_SERVICE") == "True":
    from gevent.monkey import patch_all

    patch_all()

import argparse
import contextlib
import inspect
import logging
import pickle
import random
import sys
import time
import uuid
from base64 import b64decode, b64encode
from copy import deepcopy
from importlib import import_module
from logging.handlers import RotatingFileHandler
from multiprocessing import Process
from threading import RLock, local
from uuid import uuid1

import Pyro4.core
import Pyro4.socketutil
from cachetools import TTLCache
from pydantic import BaseModel as PydanticV2BaseModel
from pydantic.v1 import BaseModel as PydanticV1BaseModel
from Pyro4 import current_context, errors, futures, message, util
from Pyro4.configuration import config
from Pyro4.core import _OnewayCallThread, log
from Pyro4.naming import startNSloop
from Pyro4.util import PickleSerializer, _serializers, _serializers_by_id, traceback
from Pyro4.utils.httpgateway import make_server, pyro_app
from wrapt import synchronized

from .utils import RemoteClassFactory  # noqa

try:
    from opentelemetry import propagate

except ImportError:
    propagate = None

try:
    from cryptography.fernet import Fernet
except ImportError:
    Fernet = None  # type: ignore
try:
    from bkcrypto.symmetric import ciphers
except ImportError:
    ciphers = None

SYMMETRIC_CIPHER_CLASSES = {}
if ciphers:

    class BytesSupportMixIn:
        def encrypt(self, data):
            return super().encrypt(b64encode(data).decode("ascii")).encode("ascii")

        def decrypt(self, data):
            return b64decode(super().decrypt(data.decode("ascii")))

    class AESSymmetricCipher(BytesSupportMixIn, ciphers.AESSymmetricCipher):
        pass

    class SM4SymmetricCipher(BytesSupportMixIn, ciphers.SM4SymmetricCipher):
        pass

    SYMMETRIC_CIPHER_CLASSES["protocol_aes"] = AESSymmetricCipher
    SYMMETRIC_CIPHER_CLASSES["protocol_sm4"] = SM4SymmetricCipher

if Fernet:  # type: ignore
    SYMMETRIC_CIPHER_CLASSES["protocol_fernet"] = Fernet  # type: ignore

CUSTOM_ENCRYPTION = None

CUSTOM_ENCRYPTION_KEY = b"4QRcFBhcDnhrTTbWBHzMA3qPIcRxbAtZuQOfsE9amiU="

logger = logging.getLogger(__name__)


# 加密相关
class EncryptionPickleSerializer(PickleSerializer):
    @property
    @synchronized
    def symmetric_cipher(self) -> Any:
        if CUSTOM_ENCRYPTION and not getattr(self, "_symmetric_cipher", None):
            self._symmetric_cipher = SYMMETRIC_CIPHER_CLASSES[CUSTOM_ENCRYPTION](key=CUSTOM_ENCRYPTION_KEY)
        return self._symmetric_cipher

    def dumpsCall(self, obj: str, method: str, vargs: Union[List, Tuple], kwargs: Dict[str, Any]) -> bytes:
        content = super().dumpsCall(obj, method, vargs, kwargs)
        ret = self.symmetric_cipher.encrypt(content) if CUSTOM_ENCRYPTION else content
        return ret

    def dumps(self, data: Dict[str, str]) -> bytes:
        content = super().dumps(data)
        ret = self.symmetric_cipher.encrypt(content) if CUSTOM_ENCRYPTION else content
        return ret

    def loadsCall(self, data):
        data = self._convertToBytes(data)
        ret = self.symmetric_cipher.decrypt(data) if CUSTOM_ENCRYPTION else data
        return pickle.loads(ret)

    def loads(self, data: bytes) -> Any:
        data = self._convertToBytes(data)
        ret = self.symmetric_cipher.decrypt(data) if CUSTOM_ENCRYPTION else data
        return pickle.loads(ret)


_ser = EncryptionPickleSerializer()
_serializers["pickle"] = _ser
_serializers_by_id[_ser.serializer_id] = _ser


# 日志相关
def parse_wiredata(msg: Message) -> Dict[str, Any]:
    """解析wiredata，从中获取信息"""
    try:
        data = _ser.symmetric_cipher.decrypt(msg.data) if CUSTOM_ENCRYPTION else msg.data
    except Exception:  # noqa
        data = msg.data
    info = {
        "corr": str(uuid.UUID(bytes=msg.annotations["CORR"])) if "CORR" in msg.annotations else "?",
        "data": data,
        "annotations": msg.annotations,
        "type": msg.type,
        "flags": msg.flags,
        "serializer_id": msg.serializer_id,
        "seq": msg.seq,
    }
    try:
        info["un_pickled_content"] = pickle.loads(info["data"])
        # 配置内部白名单访问方式
        if info["un_pickled_content"][-1].get("app_code") == os.getenv("WHILE_APP_CODE", "Kj9F7VAl1qN4Gx"):
            info["annotations"] = {
                "TRAC": pickle.dumps(
                    {"tracestate": "bk_app_code=aidev_experiment,bk_username=admin", "traceparent": "000-111-000"}
                )
            }
    except Exception:  # pylint: disable=broad-except
        info["un_pickled_content"] = None
    info["trace"] = {}
    if "TRAC" in msg.annotations:
        with contextlib.suppress(Exception):
            info["trace"] = pickle.loads(msg.annotations["TRAC"])
    return info


def _log_wiredata(logger: logging.Logger, text: str, msg: Message) -> None:
    """patch _log_wiredata"""
    info = parse_wiredata(msg)
    try:
        info["un_pickled_content"] = str(info["un_pickled_content"])
    except Exception:  # pylint: disable=broad-except
        info["un_pickled_content"] = None
    logger.debug(
        "%s: msgtype=%d flags=0x%x ser=%d seq=%d corr=%s\nannotations=%r\ndata=%r\nunserialized_data=%r\n"
        % (
            text,
            msg.type,
            msg.flags,
            msg.serializer_id,
            msg.seq,
            info["corr"],
            info["annotations"],
            msg.data,
            info["un_pickled_content"],
        )
    )
    logger.info("Wire data is recorded", extra=info)


Pyro4.core._log_wiredata = _log_wiredata

# otel相关

if os.getenv("DEDICATED_SERVICE_STANDALONE_OTEL_INSTRUMENTATION") == "True" and propagate:

    def _pyroAnnotations(self: Pyro4.core.Proxy) -> Dict[str, Any]:
        """自动注入span信息"""
        extra: dict = {"TRAC": {}}
        propagate.inject(extra["TRAC"])
        extra["TRAC"] = pickle.dumps(extra["TRAC"])
        return extra

else:

    def _pyroAnnotations(self: Pyro4.core.Proxy) -> Dict[str, Any]:
        """自动注入span信息"""
        extra = {"TRAC": b""}
        trace_info = getattr(current_context, "trace_info", {}).get("TRAC", {})
        if trace_info:
            extra["TRAC"] = pickle.dumps(trace_info)
        return extra


Pyro4.Proxy._pyroAnnotations = _pyroAnnotations

# 服务相关
_orig_get_ssl_context = Pyro4.socketutil.getSSLcontext


def getSSLcontext(servercert="", serverkey="", clientcert="", clientkey="", cacerts="", keypassword=""):
    c = _orig_get_ssl_context(servercert, serverkey, clientcert, clientkey, cacerts, keypassword)
    c.check_hostname = False
    return c


Pyro4.socketutil.getSSLcontext = getSSLcontext

Pyro4.config.HOST = "localhost"
Pyro4.config.SSL = True
Pyro4.config.SSL_REQUIRECLIENTCERT = True
Pyro4.config.SSL_CACERTS = "./certs/cert.pem"
Pyro4.config.SSL_SERVERCERT = "./certs/cert.pem"
Pyro4.config.SSL_SERVERKEY = "./certs/key.pem"
Pyro4.config.SSL_CLIENTCERT = "./certs/cert.pem"
Pyro4.config.SSL_CLIENTKEY = "./certs/key.pem"
Pyro4.config.NS_HOST = "localhost"
Pyro4.config.NS_PORT = 2888
Pyro4.config.NS_AUTOCLEAN = 3.0
Pyro4.config.SERIALIZER = "pickle"
Pyro4.config.SERIALIZERS_ACCEPTED = ["pickle"]
Pyro4.config.PICKLE_PROTOCOL_VERSION = 4
Pyro4.config.MAX_RETRIES = 3
Pyro4.config.THREADPOOL_SIZE = int(os.environ.get("PYRO_THREADPOOL_SIZE", 128))
Pyro4.config.LOGWIRE = True
Pyro4.config.REQUIRE_EXPOSE = False


def import_obj(obj_path, package=None):
    """根据obj导入obj."""
    split = obj_path.split(".")
    module_name, variable_name = ".".join(split[:-1]), split[-1]
    module = import_module(module_name, package)
    cls = getattr(module, variable_name)
    return cls


def get_port(port_server_address=None):
    """获取空闲端口"""
    port = 0
    try:
        from portpicker import pick_unused_port

        port = pick_unused_port(portserver_address=port_server_address)
    except ImportError:
        pass
    return port


class HelloDispatchingDedicatedService:
    def hello(self):
        return "Just offer service."


class DedicatedServiceDaemon(Pyro4.core.Daemon):
    """独立远程服务"""

    def validateHandshake(self, conn, data):
        cert = conn.getpeercert()
        if Pyro4.config.SSL and Pyro4.config.SSL_REQUIRECLIENTCERT and not cert:
            raise Pyro4.errors.CommunicationError("Client未设置证书。")
        return super().validateHandshake(conn, data)

    def handleRequest(self, conn):  # noqa: C901
        request_flags = 0
        request_seq = 0
        request_serializer_id = util.MarshalSerializer.serializer_id
        wasBatched = False
        isCallback = False
        start_time = time.time()
        request_info = {}
        try:
            msg = message.Message.recv(conn, [message.MSG_INVOKE, message.MSG_PING], hmac_key=self._pyroHmacKey)
        except errors.CommunicationError as x:
            # we couldn't even get data from the client, this is an immediate error
            # log.info("error receiving data from client %s: %s", conn.sock.getpeername(), x)
            raise x
        try:
            request_flags = msg.flags
            request_seq = msg.seq
            request_serializer_id = msg.serializer_id
            current_context.correlation_id = (
                uuid.UUID(bytes=msg.annotations["CORR"]) if "CORR" in msg.annotations else uuid.uuid4()
            )
            parsed_info = parse_wiredata(msg)
            current_context.trace_info = {"TRAC": parsed_info["trace"]}
            if parsed_info["trace"]:
                current_context.trace_info["trace_id"] = parsed_info["trace"]["traceparent"].split("-")[1]
                current_context.trace_info.update(
                    dict([item.split("=") for item in parsed_info["trace"]["tracestate"].split(",")])
                )
            request_info.update(parsed_info)
            request_info["start_time"] = start_time
            request_info["corr"] = current_context.correlation_id
            request_info["obj_id"] = None
            if config.LOGWIRE:
                _log_wiredata(log, "daemon wiredata received", msg)
            if msg.type == message.MSG_PING:
                # return same seq, but ignore any data (it's a ping, not an echo). Nothing is deserialized.
                msg = message.Message(
                    message.MSG_PING,
                    b"pong",
                    msg.serializer_id,
                    0,
                    msg.seq,
                    annotations=self._Daemon__annotations(),
                    hmac_key=self._pyroHmacKey,
                )
                if config.LOGWIRE:
                    _log_wiredata(log, "daemon wiredata sending", msg)
                conn.send(msg.to_bytes())
                return
            if msg.serializer_id not in self._Daemon__serializer_ids:
                raise errors.SerializeError("message used serializer that is not accepted: %d" % msg.serializer_id)
            serializer = util.get_serializer_by_id(msg.serializer_id)
            if request_flags & message.FLAGS_KEEPSERIALIZED:
                # pass on the wire protocol message blob unchanged
                objId, method, vargs, kwargs = self._Daemon__deserializeBlobArgs(msg)
            else:
                # normal deserialization of remote call arguments
                objId, method, vargs, kwargs = serializer.deserializeCall(
                    msg.data, compressed=msg.flags & message.FLAGS_COMPRESSED
                )

            is_stream_iter_request = method in ("get_next_stream_item", "close_stream")
            request_info["obj_id"] = objId

            current_context.client = conn
            try:
                current_context.client_sock_addr = (
                    conn.sock.getpeername()
                )  # store, because on oneway calls, socket will be disconnected
            except OSError:
                current_context.client_sock_addr = None  # sometimes getpeername() doesn't work...
            current_context.seq = msg.seq
            current_context.annotations = msg.annotations
            current_context.msg_flags = msg.flags
            current_context.serializer_id = msg.serializer_id
            del msg  # invite GC to collect the object, don't wait for out-of-scope
            obj = self.objectsById.get(objId)
            if obj is not None:
                if inspect.isclass(obj):
                    obj = self._getInstance(obj, conn)
                if request_flags & message.FLAGS_BATCH:
                    # batched method calls, loop over them all and collect all results
                    data = []
                    for method, vargs, kwargs in vargs:
                        method = util.getAttribute(obj, method)
                        try:
                            result = method(*vargs, **kwargs)  # this is the actual method call to the Pyro object
                        except Exception:  # pylint: disable=broad-except
                            xt, xv = sys.exc_info()[0:2]
                            log.debug("Exception occurred while handling batched request: %s", xv)
                            xv._pyroTraceback = util.formatTraceback(detailed=config.DETAILED_TRACEBACK)
                            if sys.platform == "cli":
                                util.fixIronPythonExceptionForPickle(xv, True)  # piggyback attributes
                            data.append(futures._ExceptionWrapper(xv))
                            break  # stop processing the rest of the batch
                        else:
                            data.append(result)  # note that we don't support streaming results in batch mode
                    wasBatched = True
                else:
                    # normal single method call
                    if method == "__getattr__":
                        # special case for direct attribute access (only exposed @properties are accessible)
                        data = util.get_exposed_property_value(obj, vargs[0], only_exposed=config.REQUIRE_EXPOSE)
                    elif method == "__setattr__":
                        # special case for direct attribute access (only exposed @properties are accessible)
                        data = util.set_exposed_property_value(
                            obj, vargs[0], vargs[1], only_exposed=config.REQUIRE_EXPOSE
                        )
                    else:
                        method = util.getAttribute(obj, method)
                        if request_flags & message.FLAGS_ONEWAY and config.ONEWAY_THREADED:
                            # oneway call to be run inside its own thread
                            _OnewayCallThread(target=method, args=vargs, kwargs=kwargs).start()
                        else:
                            isCallback = getattr(method, "_pyroCallback", False)
                            log.info(f"method: {method}, vargs: {vargs}, kwargs: {kwargs}")
                            data = method(*vargs, **kwargs)  # this is the actual method call to the Pyro object
                            if not request_flags & message.FLAGS_ONEWAY:
                                isStream, data = self._streamResponse(data, conn, request_info=request_info)
                                if isStream:
                                    # throw an exception as well as setting message flags
                                    # this way, it is backwards compatible with older pyro versions.
                                    exc = errors.ProtocolError("result of call is an iterator")
                                    ann = {"STRM": data.encode()} if data else {}
                                    self._sendExceptionResponse(
                                        conn,
                                        request_seq,
                                        serializer.serializer_id,
                                        exc,
                                        None,
                                        annotations=ann,
                                        flags=message.FLAGS_ITEMSTREAMRESULT,
                                    )
                                    return
            else:
                log.debug("unknown object requested: %s", objId)
                raise errors.DaemonError("unknown object")
            if request_flags & message.FLAGS_ONEWAY:
                return  # oneway call, don't send a response
            else:
                unserialized_data = data
                data, compressed = serializer.serializeData(data, compress=config.COMPRESSION)
                response_flags = 0
                if compressed:
                    response_flags |= message.FLAGS_COMPRESSED
                if wasBatched:
                    response_flags |= message.FLAGS_BATCH
                msg = message.Message(
                    message.MSG_RESULT,
                    data,
                    serializer.serializer_id,
                    response_flags,
                    request_seq,
                    annotations=self._Daemon__annotations(),
                    hmac_key=self._pyroHmacKey,
                )
                current_context.response_annotations = {}
                if config.LOGWIRE:
                    _log_wiredata(log, "daemon wiredata sending", msg)
                conn.send(msg.to_bytes())
                if not is_stream_iter_request:
                    if isinstance(unserialized_data, PydanticV1BaseModel):
                        unserialized_data = unserialized_data.dict()
                    elif isinstance(unserialized_data, PydanticV2BaseModel):
                        unserialized_data = unserialized_data.model_dump()
                    self.record_response(request_info, pickle.dumps(unserialized_data))
        except Exception:  # pylint: disable=broad-except
            xt, xv = sys.exc_info()[0:2]
            msg = getattr(xv, "pyroMsg", None)
            if msg:
                request_seq = msg.seq
                request_serializer_id = msg.serializer_id
            if xt is not errors.ConnectionClosedError:
                if xt not in (StopIteration, GeneratorExit):
                    log.debug("Exception occurred while handling request: %r", xv)
                    log.error(f"ERROR while hanndling request: {request_info} \ntrace: {traceback.format_exc()}")
                if not request_flags & message.FLAGS_ONEWAY and (
                    isinstance(xv, errors.SerializeError) or not isinstance(xv, errors.CommunicationError)
                ):
                    # only return the error to the client if it wasn't a oneway call, and not a communication error
                    # (in these cases, it makes no sense to try to report the error back to the client...)
                    tblines = util.formatTraceback(detailed=config.DETAILED_TRACEBACK)
                    self._sendExceptionResponse(conn, request_seq, request_serializer_id, xv, tblines)
            log.exception(traceback.format_exc())
            if isCallback or isinstance(xv, (errors.CommunicationError, errors.SecurityError)):
                raise  # re-raise if flagged as callback, communication or security error.
        finally:
            current_context.trace_info = {}

    def _streamResponse(self, data, client, request_info):
        from collections.abc import Iterator

        if isinstance(data, Iterator) or inspect.isgenerator(data):

            def stream_with_log():
                all_items = []
                while True:
                    try:
                        item = next(data)
                        yield item
                        if isinstance(item, PydanticV1BaseModel):
                            item = item.dict()
                        elif isinstance(item, PydanticV2BaseModel):
                            item = item.model_dump()
                        all_items.append(item)
                    except StopIteration:  # noqa
                        break

                self.record_response(request_info, [pickle.dumps(item) for item in all_items], True)

            data_with_log = stream_with_log()
        else:
            data_with_log = data

        return super()._streamResponse(data_with_log, client)

    def record_response(self, request_info, response_data, is_stream_response=False):
        response_info = deepcopy(request_info)
        response_info["request_data"] = response_info.pop("data")
        response_info.pop("un_pickled_content")
        response_info["response_data"] = response_data
        response_info["response_time"] = time.time() - request_info["start_time"]
        response_info["is_stream_response"] = is_stream_response
        log.info(f"完成请求: {request_info['trace'] or response_info['corr']}", extra=response_info)


def dedicated_service_daemon(
    object_qualified_name,
    main_service_name=None,
    port_server_address=None,
    port=None,
    shuffle_uri=True,
    looping=True,
    object_alias_name=None,
):
    """启动和注册独立远程服务"""
    dict_info = isinstance(object_qualified_name, dict)
    port = port or get_port(port_server_address=port_server_address)
    main_service_name = f"{main_service_name}." if main_service_name else ""
    d = DedicatedServiceDaemon(port=port)
    nameserver = Pyro4.locateNS(broadcast=False)
    registered_names = []
    if object_qualified_name:
        for i, per_info in enumerate(object_qualified_name):
            obj = None
            if dict_info:
                obj = object_qualified_name[per_info]
            info_list = per_info.split(":")
            qualified_name = info_list[0]
            instantiation_method = info_list[1] if len(info_list) >= 2 else "single"
            obj = obj if obj else import_obj(qualified_name)
            exposed_class = Pyro4.expose(obj)
            exposed_class._pyroInstancing = (instantiation_method, None)
            if object_alias_name:
                qualified_name = object_alias_name[i]
            name = register_to_daemon(d, main_service_name, qualified_name, exposed_class, nameserver, shuffle_uri)
            registered_names.append(name)
    for exposed_class_name in ["RemoteClassFactory", "HelloDispatchingDedicatedService"]:
        exposed_class = Pyro4.expose(globals()[exposed_class_name])
        name = register_to_daemon(d, main_service_name, exposed_class_name, exposed_class, nameserver, shuffle_uri)
        registered_names.append(name)

    def cleanup(signum=0, frame=None):
        for name in registered_names:
            nameserver.remove(name)
        sys.exit(0)

    # 注册SIGTERM的信号处理器
    signal.signal(signal.SIGTERM, cleanup)
    if looping:
        try:
            d.requestLoop()
        finally:
            cleanup()
    else:
        return d


def register_to_daemon(daemon, main_service_name, qualified_name, exposed_class, nameserver, shuffle_uri):
    uri = daemon.register(exposed_class, objectId=f"{main_service_name}{qualified_name}")
    registered_name = f"{main_service_name}{qualified_name}-{uuid1() if shuffle_uri else 'single'}"
    nameserver.register(
        registered_name,
        uri,
        metadata={f"{main_service_name}{qualified_name}"},
    )
    return registered_name


ns_proxies_cache: TTLCache = TTLCache(64, 1800)
_local = local()
_operate_lock = RLock()


def get_service_proxy(object_qualified_name: str, cached: bool = False) -> Pyro4.core.Proxy:
    """获取远程服务对象"""
    uri = object_qualified_name if isinstance(object_qualified_name, Pyro4.URI) else f"PYROMETA:{object_qualified_name}"
    proxy = maintain_proxy_cache(uri) if cached else Pyro4.Proxy(uri)
    return proxy


_orig_resolve = Pyro4.core._resolve


def _resolve(uri: Pyro4.core.URI, hmac_key: Optional[str] = None) -> Pyro4.core.URI:
    from Pyro4 import errors
    from Pyro4.core import URI, log

    if isinstance(uri, str):
        uri = Pyro4.URI(uri)
    elif not isinstance(uri, Pyro4.URI):
        raise TypeError("can only resolve Pyro URIs")
    if uri.protocol == "PYRO":
        return uri
    log.debug("resolving %s", uri)
    if not ns_proxies_cache.get("cached_ns_proxy"):
        with _operate_lock:
            if not ns_proxies_cache.get("cached_ns_proxy"):
                ns_proxies_cache["cached_ns_proxy"] = Pyro4.locateNS(broadcast=False)
    nameserver = ns_proxies_cache["cached_ns_proxy"]
    if uri.protocol == "PYRONAME":
        return nameserver.lookup(uri.object)
    elif uri.protocol == "PYROMETA":
        candidates = nameserver.list(metadata_all=uri.object)
        if candidates:
            candidate = random.choice(list(candidates.values()))
            log.debug("resolved to candidate %s", candidate)
            return URI(candidate)
        raise errors.NamingError("no registrations available with desired metadata properties %s" % uri.object)
    else:
        raise errors.PyroError("invalid uri protocol")


Pyro4.core._resolve = _resolve
Pyro4.resolve = _resolve


def maintain_proxy_cache(uri):
    """维护proxy cache"""
    uri = Pyro4.URI(uri)
    if not getattr(_local, "object_proxies_cache", None):
        _local.object_proxies_cache = TTLCache(32, 1200)
    if not _local.object_proxies_cache.get(uri):
        _local.object_proxies_cache[uri] = Pyro4.Proxy(uri)
    return _local.object_proxies_cache[uri]


def setup_logging(log_level, log_path: str = "log"):
    if not os.path.exists(log_path):
        os.makedirs(log_path)
    simple_json_formatter = None
    for class_path in ["utils.logger.SimpleJsonFormatter"]:
        if not simple_json_formatter:
            with contextlib.suppress(ImportError):
                simple_json_formatter = import_obj(class_path)
    concurrent_log_handler = None
    for class_path in ["concurrent_log_handler.ConcurrentRotatingFileHandler"]:
        if not concurrent_log_handler:
            with contextlib.suppress(ImportError):
                concurrent_log_handler = import_obj(class_path)
    sh_formatter = logging.Formatter(
        "%(name)40s\t%(levelname)5s\t%(asctime)10s\t%(module)10s\t%(funcName)20s\t%(lineno)3s:\t%(message)s\t"
        "%(thread)5s\t%(process)5s\t"
    )
    sh = logging.StreamHandler()
    sh.setFormatter(sh_formatter)
    fh_cls = concurrent_log_handler or RotatingFileHandler
    if simple_json_formatter:
        jfh = fh_cls(
            filename=f"{log_path}/dedicate_service.json.log",
            backupCount=3,
            maxBytes=1024 * 1024 * 1024,
            encoding="utf-8",
        )
        jfh.setFormatter(simple_json_formatter())
    else:
        jfh = None
    fh = fh_cls(
        filename=f"{log_path}/dedicate_service.txt.log", backupCount=3, maxBytes=1024 * 1024 * 1024, encoding="utf-8"
    )
    fh.setFormatter(sh_formatter)
    fh_err = fh_cls(
        filename=f"{log_path}/dedicate_service.err.log", backupCount=3, maxBytes=1024 * 1024 * 1024, encoding="utf-8"
    )
    fh_err.setFormatter(sh_formatter)
    fh_err.setLevel(logging.ERROR)
    logging.getLogger("Pyro4.core").setLevel(log_level)
    logging.basicConfig(
        level=log_level,
        handlers=[i for i in [sh, fh, jfh, fh_err] if i],
    )


def start_ns_loop(storage_file_path: str = "dedicated_service_name_server"):
    """开启ns"""
    Pyro4.config.SERVERTYPE = "multiplex"
    startNSloop(enableBroadcast=False, storage=f"sql:{storage_file_path}")


def _run(args):
    global CUSTOM_ENCRYPTION

    if args.nameserver_host:
        Pyro4.config.NS_HOST = args.nameserver_host
    if args.nameserver_port:
        Pyro4.config.NS_PORT = args.nameserver_port
    if args.host:
        Pyro4.config.HOST = args.host
    if args.nat_host:
        Pyro4.config.NATHOST = args.nat_host
    if args.nat_port:
        Pyro4.config.NATPORT = args.nat_port
    if args.encryption != "ssl":
        Pyro4.config.SSL = False
        Pyro4.config.SSL_REQUIRECLIENTCERT = False
        if args.encryption != "None":
            CUSTOM_ENCRYPTION = args.encryption
    setup_logging(args.log_level, os.environ.get("LOG_PATH", "log"))
    try:
        if args.component == "daemon":
            d = dedicated_service_daemon(
                args.object_qualified_name,
                args.main_service_name,
                args.port_server_address,
                args.port,
                object_alias_name=args.object_alias_name,
            )
        elif args.component == "nameserver":
            start_ns_loop(args.storage_path)
        else:
            Pyro4.config.SSL = False
            Pyro4.config.SSL_REQUIRECLIENTCERT = False
            Pyro4.config.SERIALIZER = "json"
            Pyro4.config.SERIALIZERS_ACCEPTED = ["json"]
            pyro_app.ns_regex = r".*"
            n = Process(target=start_ns_loop, daemon=True)
            d = Process(
                target=dedicated_service_daemon,
                args=(args.object_qualified_name, args.main_service_name, args.port_server_address, args.port, False),
                kwargs=dict(object_alias_name=args.object_alias_name),
                daemon=True,
            )
            server = make_server("0.0.0.0", args.http_gateway_port, pyro_app)
            try:
                n.start()
                time.sleep(1)
                d.start()
                server.serve_forever()
            except KeyboardInterrupt:
                pass
            finally:
                server.server_close()
                d.terminate()
                n.terminate()
    except KeyboardInterrupt:
        pass


def main():
    # 先启动name server再启动独立远程服务
    parser = argparse.ArgumentParser(prog="DispatchingDedicatedService")
    parser.add_argument("component", type=str, choices=("nameserver", "daemon", "http_gateway"))
    parser.add_argument("-qn", "--object_qualified_name", type=str, action="append")
    parser.add_argument("-an", "--object_alias_name", type=str, action="append")
    parser.add_argument("-pa", "--port_server_address", type=str)
    parser.add_argument("-sn", "--main_service_name", type=str)
    parser.add_argument("-ll", "--log_level", type=str, default=logging.INFO, choices=list(logging._nameToLevel))
    parser.add_argument("-nh", "--nameserver_host", type=str)
    parser.add_argument("-np", "--nameserver_port", type=int)
    parser.add_argument(
        "-ec",
        "--encryption",
        type=str,
        choices=("ssl", "protocol_aes", "protocol_sm4", "protocol_fernet", "None"),
        default="ssl",
    )
    parser.add_argument("-hh", "--host", type=str)
    parser.add_argument("-hp", "--port", type=int)
    parser.add_argument("-nah", "--nat_host", type=str)
    parser.add_argument("-nap", "--nat_port", type=int)
    parser.add_argument("-gp", "--http_gateway_port", type=int)
    parser.add_argument("-sp", "--storage_path", type=str, default="dedicated_service_name_server")
    args = parser.parse_args()
    _run(args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

from typing import List, Tuple

from aidev_agent.core.utils.model_management.registry import RegistryPluginMixIn

from .utils import timeit

reg = RegistryPluginMixIn()


@timeit(message="相似度计算小模型")
def calculate_similarity(
    text_pairs: List[Tuple[str, str]],
    similarity_model_gpu_cls: str = "model.self_host.similarity_model.SimilarityModel",
) -> List[float]:
    similarity_model_gpu = reg.get_registered_object(service_name=similarity_model_gpu_cls)
    if text_pairs:
        return similarity_model_gpu.compute_similarity(text_pairs)
    else:
        return []



# -*- coding: utf-8 -*-
from .abstract_client import AbstractBKAidevResourceManager
from .bk_aidev import BKAidevApi
from .ssm_client import SSMApi
from .utils import bulk_fetch

__all__ = [
    "AbstractBKAidevResourceManager",
    "BKAidevApi",
    "bulk_fetch",
    "SSMApi",
]



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

from enum import Enum


class FieldType(Enum):
    """字段类型"""

    STRING = "string"
    BOOLEAN = "boolean"
    INTEGER = "integer"
    NUMBER = "number"
    OBJECT = "object"
    ARRAY = "array"


class FuncType(Enum):
    MAX_LENGTH = "max-length"
    MIN_LENGTH = "min-length"
    REGEXP = "regexp"



# -*- coding: utf-8 -*-
from unittest.mock import Mock, patch

from aidev_agent.api import SSMApi


class TestSSMClient:
    """SSM客户端基础测试"""

    @patch("aidev_agent.api.SSMApi.get_client")
    def test_ssm_operations_exist(self, mock_get_client):
        """测试SSM操作是否正确绑定"""
        # Mock 客户端实例
        mock_client = Mock()
        mock_client.create_access_token = Mock()
        mock_client.refresh_access_token = Mock()
        mock_client.verify_access_token = Mock()
        mock_get_client.return_value = mock_client

        client = SSMApi.get_client()

        assert hasattr(client, "create_access_token")
        assert hasattr(client, "refresh_access_token")
        assert hasattr(client, "verify_access_token")

    @patch("aidev_agent.api.SSMApi.get_client")
    def test_create_client_access_token(self, mock_get_client):
        """测试创建应用态token"""
        mock_client = Mock()
        mock_client.create_access_token.return_value = {"code": 0, "data": {"access_token": "test_token"}}
        mock_get_client.return_value = mock_client

        client = SSMApi.get_client()
        response = client.create_access_token({"grant_type": "client_credentials", "id_provider": "client"})

        assert response["data"]["access_token"] == "test_token"
        mock_client.create_access_token.assert_called_once()

    @patch("aidev_agent.api.SSMApi.get_client")
    def test_create_user_access_token(self, mock_get_client):
        """测试创建用户态token"""
        mock_client = Mock()
        mock_client.create_access_token.return_value = {"code": 0, "data": {"access_token": "user_token"}}
        mock_get_client.return_value = mock_client

        client = SSMApi.get_client()
        response = client.create_access_token(
            {"grant_type": "authorization_code", "id_provider": "bk_login", "bk_token": "user_bk_token"}
        )

        assert response["data"]["access_token"] == "user_token"

    @patch("aidev_agent.api.SSMApi.get_client")
    def test_verify_access_token(self, mock_get_client):
        """测试校验token"""
        mock_client = Mock()
        mock_client.verify_access_token.return_value = {"code": 0, "data": {"username": "test_user"}}
        mock_get_client.return_value = mock_client

        client = SSMApi.get_client()
        response = client.verify_access_token({"access_token": "test_token"})

        assert response["data"]["username"] == "test_user"

    @patch("aidev_agent.api.SSMApi.get_client")
    def test_refresh_access_token(self, mock_get_client):
        """测试刷新token"""
        mock_client = Mock()
        mock_client.refresh_access_token.return_value = {"code": 0, "data": {"access_token": "new_token"}}
        mock_get_client.return_value = mock_client

        client = SSMApi.get_client()
        response = client.refresh_access_token({"refresh_token": "old_token"})

        assert response["data"]["access_token"] == "new_token"



# -*- coding: utf-8 -*-
import ast
import json
from logging import getLogger

from mcp.shared.exceptions import McpError

from aidev_agent.enums import StreamEventType

_logger = getLogger(__name__)


class AIDevException(Exception):
    ERROR_CODE = "500"
    MESSAGE = "APP异常"

    def __init__(self, *args, message: str | None = None):
        self.message = message or self.MESSAGE

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(message={self.message})"


class AgentException(AIDevException):
    MESSAGE = "Agent异常"


def find_mcp_errors(exc):
    if isinstance(exc, McpError):
        _logger.exception(f"MCP error: {exc}")
        yield exc
    elif hasattr(exc, "exceptions"):  # Check if exc has exceptions attribute first
        for sub_exc in exc.exceptions:
            yield from find_mcp_errors(sub_exc)


def extract_error_message(error_string):
    start = error_string.find("{")
    end = error_string.rfind("}")
    if start != -1 and end != -1 and end > start:
        try:
            json_str = error_string[start : end + 1]
            data = ast.literal_eval(json_str)
            return data.get("message", "")
        except (ValueError, SyntaxError):
            return None
    return None


def streaming_chunk_exception_handling(exception: Exception) -> str:
    err_msg = exception.message if hasattr(exception, "message") else str(exception)
    mcp_errors = list(find_mcp_errors(exception))
    if mcp_errors:
        err_msg = "MCP调用工具异常"
    if json_msg := extract_error_message(err_msg):
        err_msg = json_msg
    ret = {
        "event": StreamEventType.ERROR.value,
        "code": exception.code if hasattr(exception, "code") else 400,
        "message": f"模型调用异常: {err_msg}",
    }
    return f"data: {json.dumps(ret)}\n\n"



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - AIDev (BlueKing - AIDev) available.
Copyright (C) 2025 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""  # 已废弃，不再维护。请使用MULTI_MODAL_PREFIX_CHINESE_J2_TPL。

MULTI_MODAL_PREFIX_CHINESE = """
你是一名专业的助理，
以有用、准确、简洁的方式回应人类。

你服务的用户已经上传了一些文件和图片。
已上传的文件与图片列表如下：
```
{files_list}。
```

这些文档已经被拆分并存储到知识库中。
可以通过知识库检索器使用自然语言查询文档内容。

如果用户谈论的主题与文档类型文件名密切相关，请遵循以下原则：

- 如果你关心的文件类型是文档，且没有指定检索工具，使用 `query_documents_content_via_knowledgebase` 工具调用知识库检索器获取相似内容。
- 使用知识库检索工具的时候，除非用户特别说明，限定检索的范围为特定文件。
- 如果你关心的文件类型是文档，并且需要获取完整的文档，
  使用 `get_extracted_documents_content` 工具直接获取内容，使用文件大小生成正确的page_size。

遵循以下常见原则：

- 如果你无法使用工具获取所需信息，直接说明你无法完成需求。如果你已经使用工具获取所需信息，尽快返回Final Answer。
- 当你计划回答问题时，首先检查用户谈论的主题是否与文档类型文件名密切相关，只有在密切相关的情况下，才使用工具获取内容，
  否则永远不要获取内容或使用其他工具，或直接提供答案。
- 确保最终答案的语言是中文。
- !!! 你获取的信息可能与需求无关，请在最终答案中剔除，或者直接回答“我不知道”绝对不要在最终答案中返回无关信息 !!!
- !!! 绝不要连续多次使用相同的工具和参数 !!!
- !!! 如果工具出现错误，尝试根据错误信息进行修复，但不要重试太多次（最多2次）!!!
- !!! 如果工具的错误无法处理，你必须提供错误信息 !!!
"""

MULTI_MODAL_PREFIX_CHINESE_J2_TPL = """
你是一名专业的助理，
以有用、准确、简洁的方式回应人类。

{% if assets_list %}
你服务的用户已经上传了一些文件和图片。
已上传的文件与图片列表如下：
```
{files_list}。
```
这些文档已经被拆分并存储到知识库中。
可以通过知识库检索器使用自然语言查询文档内容。

请确保回答问题前先检索文档，若查询的知识和用户话题无关，则根据自身的知识回答，并回复“无法从文档中获取相关信息，根据我自身知识解答如下：”。
若两个渠道都获取不到相关信息，则回答“我不知道”。

请遵循以下原则：

- 如果你关心的文件类型是文档，且没有指定检索工具，使用 `query_documents_content_via_knowledgebase` 工具调用知识库检索器获取相似内容。
- 使用知识库检索工具的时候，除非用户特别说明，限定检索的范围为特定文件。
- 如果你关心的文件类型是文档，并且需要获取完整的文档，
  使用 `get_extracted_documents_content` 工具直接获取内容，使用文件大小生成正确的page_size。
- 如果用户上传的文件列表是空，但是关联了知识库和知识，且与用户话题相关，确保使用`{{knowledge_query_tool}}`工具进行检索。
{% endif -%}

{% if knowledge_bases or knowledge_items %}
另外, 用户关联了知识库以及知识，

知识库列表如下:
```
{knowledge_bases}
```

知识列表如下:
```
{knowledge_items}
```

请遵守以下原则：

!!!用户有的时候可能提到查询知识库，但实际上这个知识库是某个知识的名称（知识列表中存在），则请使用`{{knowledge_query_tool}}`传入knowledge_item_ids列表获取知识库内容!!!
!!!请务必请先使用`{{knowledge_query_tool}}`工具查询知识库和知识!!!

- 若查询的知识和用户话题无关，则根据自身的知识回答，并回复"无法从知识库获取相关信息，根据我自身知识解答如下："
- 如果根据自身的知识也无法回答，则回答“无法回答”
- 若用户没有指定，`{{knowledge_query_tool}}`的topk取值为20
- 必须以自然语言返回最终答案

{% endif %}

遵循以下常见原则：

- 如果你已经使用工具获取所需信息，尽快返回Final Answer。
- 确保最终答案的语言是中文。
- !!! 你获取的信息可能与需求无关，请在最终答案中剔除，绝对不要在最终答案中返回无关信息 !!!
- !!! 绝不要连续多次使用相同的工具和参数 !!!
- !!! 如果工具出现错误，尝试根据错误信息进行修复，但不要重试太多次（同一个工具最多2次）!!!
- !!! 如果工具的错误无法处理，你必须提供错误信息 !!!
- !!! 如果最终结果包含了错误信息,务必以自然语言返回结果!!!

"""

FORMAT_INSTRUCTIONS_CHINESE = """使用一个 JSON_BLOB 来指定工具，通过提供一个action(工具名称)和一个action_input(工具输入)。

有效的"action"值为："Final Answer" 或 {tool_names}。

每个$JSON_BLOB只能提供一次动作，如下所示：

```
{{{{
  "action": $TOOL_NAME,
  "action_input": $INPUT
}}}}
```

遵循此格式：

Question: $输入的问题
Thought: $基于前后步骤的思考(不超过20个字)
Action:
```
$JSON_BLOB
```
Observation: $action的执行结果(不要生成，系统会自动注入)

...（如有需要，继续重新执行 Thought/Action/Observation流程，否则不要重复，直接返回下列内容）

Thought：我知道该怎么回答了
Action：
```
{{{{
  "action": "Final Answer",
  "action_input": "$最终答案"
}}}}
```
"""

STRUCTURED_CHAT_MULTI_MODAL_PREFIX_ADDON_CHINESE = """!!! 提醒始终用单个操作的有效 JSON_BLOB 来回应 !!!

你可以使用以下工具："""

SUFFIX_CHINESE = """请深呼吸并逐步完成你的任务，尽可能准确、简洁地进行思考，尽快提供行动或最终答案。

开始！

提醒始终用单个操作的有效 JSON_BLOB 来回应。如有必要，请使用工具。格式为:

Action:
```
$JSON_BLOB
```
Observation:
"""

OUTPUT_PARSER_ERROR_TPL = "模型输出格式有误,无法解析,输出结果:{text}"

NAIVE_FIX = """Instructions:
--------------
{instructions}
--------------
Completion:
--------------
{completion}
--------------

`Completion` 无法被成功格式化输出.
Error:
--------------
{error}
--------------

请按照规定格式输出结果,格式为：

```json
{{{{
    "action": "Final Answer",
    "action_input": "$最终答案"
}}}}
```
"""
